# -----------------------------------------------------------------------------
# Copyright © 2009- The QtPy Contributors
#
# Released under the terms of the MIT License
# (see LICENSE.txt for details)
# -----------------------------------------------------------------------------

"""Provide a CLI to allow configuring developer settings, including mypy."""

# Standard library imports
import argparse
import textwrap


def print_version():
    """Print the current version of the package."""
    import qtpy
    print('QtPy version', qtpy.__version__)


def generate_mypy_args():
    """Generate a string with always-true/false args to pass to mypy."""
    options = {False: '--always-false', True: '--always-true'}

    import qtpy

    apis_active = {name: qtpy.API == name for name in qtpy.API_NAMES}
    mypy_args = ' '.join(
        f'{options[is_active]}={name.upper()}'
        for name, is_active in apis_active.items()
    )
    return mypy_args


def print_mypy_args():
    """Print the generated mypy args to stdout."""
    print(generate_mypy_args())


def generate_arg_parser():
    """Generate the argument parser for the dev CLI for QtPy."""
    parser = argparse.ArgumentParser(
        description='Features to support development with QtPy.',
    )
    parser.set_defaults(func=parser.print_help)

    parser.add_argument(
        '--version', action='store_const', dest='func', const=print_version,
        help='If passed, will print the version and exit')

    cli_subparsers = parser.add_subparsers(
        title='Subcommands', help='Subcommand to run', metavar='Subcommand')

    # Parser for the MyPy args subcommand
    mypy_args_parser = cli_subparsers.add_parser(
        name='mypy-args',
        help='Generate command line arguments for using mypy with QtPy.',
        formatter_class=argparse.RawTextHelpFormatter,
        description=textwrap.dedent(
            """
            Generate command line arguments for using mypy with QtPy.

            This will generate strings similar to the following
            which help guide mypy through which library QtPy would have used
            so that mypy can get the proper underlying type hints.

                --always-false=PYQT5 --always-false=PYQT6 --always-true=PYSIDE2 --always-false=PYSIDE6

            It can be used as follows on Bash or a similar shell:

                mypy --package mypackage $(qtpy mypy-args)
            """
        ),
    )
    mypy_args_parser.set_defaults(func=print_mypy_args)

    return parser


def main(args=None):
    """Run the development CLI for QtPy."""
    parser = generate_arg_parser()
    parsed_args = parser.parse_args(args=args)

    reserved_params = {'func'}
    cleaned_args = {key: value for key, value in vars(parsed_args).items()
                    if key not in reserved_params}
    parsed_args.func(**cleaned_args)



import contextlib
import os
import sys
import warnings

import pytest

from qtpy import PYSIDE6, PYSIDE2, QtWidgets
from qtpy.QtWidgets import QComboBox

if PYSIDE2:
    pytest.importorskip("pyside2uic", reason="pyside2uic not installed")

from qtpy import uic
from qtpy.tests.utils import not_using_conda


QCOMBOBOX_SUBCLASS = """
from qtpy.QtWidgets import QComboBox
class _QComboBoxSubclass(QComboBox):
    pass
"""

@contextlib.contextmanager
def enabled_qcombobox_subclass(temp_dir_path):
    """
    Context manager that sets up a temporary module with a QComboBox subclass
    and then removes it once we are done.
    """

    with open(
            temp_dir_path / 'qcombobox_subclass.py',
            mode='w',
            encoding="utf-8",
            ) as f:
        f.write(QCOMBOBOX_SUBCLASS)

    sys.path.insert(0, str(temp_dir_path))

    yield

    sys.path.pop(0)


@pytest.mark.skipif(
    sys.platform.startswith('linux') and not_using_conda(),
    reason="Segfaults on Linux when not using conda under all bindings (PYSIDE2/6 & PYQT5/6)")
def test_load_ui(qtbot):
    """
    Make sure that the patched loadUi function behaves as expected with a
    simple .ui file.
    """
    with warnings.catch_warnings():
        warnings.filterwarnings(
            "ignore", category=DeprecationWarning, message=".*mode.*")
        ui = uic.loadUi(os.path.join(os.path.dirname(__file__), 'test.ui'))

    assert isinstance(ui.pushButton, QtWidgets.QPushButton)
    assert isinstance(ui.comboBox, QComboBox)


@pytest.mark.skipif(
    PYSIDE2 or PYSIDE6,
    reason="PySide2uic not consistantly installed across platforms/versions")
@pytest.mark.skipif(
    sys.platform.startswith('linux') and not_using_conda(),
    reason="Segfaults on Linux when not using conda under all bindings (PYSIDE2/6 & PYQT5/6)")
def test_load_ui_type(qtbot):
    """
    Make sure that the patched loadUiType function behaves as expected with a
    simple .ui file.
    """
    with warnings.catch_warnings():
        warnings.filterwarnings(
            "ignore", category=DeprecationWarning, message=".*mode.*")
        ui_type, ui_base_type = uic.loadUiType(
            os.path.join(os.path.dirname(__file__), 'test.ui'))
    assert ui_type.__name__ == 'Ui_Form'

    class Widget(ui_base_type, ui_type):
        def __init__(self):
            super().__init__()
            self.setupUi(self)

    ui = Widget()
    assert isinstance(ui, QtWidgets.QWidget)
    assert isinstance(ui.pushButton, QtWidgets.QPushButton)
    assert isinstance(ui.comboBox, QComboBox)


@pytest.mark.skipif(
    sys.platform.startswith('linux') and not_using_conda(),
    reason="Segfaults on Linux when not using conda under all bindings (PYSIDE2/6 & PYQT5/6)")
def test_load_ui_custom_auto(qtbot, tmp_path):
    """
    Test that we can load a .ui file with custom widgets without having to
    explicitly specify a dictionary of custom widgets, even in the case of
    PySide.
    """
    with enabled_qcombobox_subclass(tmp_path):
        from qcombobox_subclass import _QComboBoxSubclass
        with warnings.catch_warnings():
            warnings.filterwarnings(
                "ignore", category=DeprecationWarning, message=".*mode.*")
            ui = uic.loadUi(
                os.path.join(os.path.dirname(__file__), 'test_custom.ui'))

    assert isinstance(ui.pushButton, QtWidgets.QPushButton)
    assert isinstance(ui.comboBox, _QComboBoxSubclass)


def test_load_full_uic():
    """Test that we load the full uic objects."""
    QT_API = os.environ.get('QT_API', '').lower()
    if QT_API.startswith('pyside'):
        assert hasattr(uic, 'loadUi')
        assert hasattr(uic, 'loadUiType')
    else:
        objects = ['compileUi', 'compileUiDir', 'loadUi', 'loadUiType',
                   'widgetPluginPath']
        assert all(hasattr(uic, o) for o in objects)



import pytest


def test_qtscxml():
    """Test the qtpy.QtScxml namespace"""
    QtScxml = pytest.importorskip("qtpy.QtScxml")

    assert QtScxml.QScxmlCompiler is not None
    assert QtScxml.QScxmlDynamicScxmlServiceFactory is not None
    assert QtScxml.QScxmlExecutableContent is not None



import pytest


def test_shiboken():
    """Test the qtpy.shiboken namespace"""
    shiboken = pytest.importorskip("qtpy.shiboken")

    assert shiboken.isValid is not None
    assert shiboken.wrapInstance is not None
    assert shiboken.getCppPointer is not None
    assert shiboken.delete is not None
    assert shiboken.dump is not None



"""Utility functions for tests."""

import os


def using_conda():
    return os.environ.get('USE_CONDA', 'Yes') == 'Yes'


def not_using_conda():
    return os.environ.get('USE_CONDA', 'No') == 'No'



import pytest


def test_qtsvgwidgets():
    """Test the qtpy.QtSvgWidgets namespace"""
    QtSvgWidgets = pytest.importorskip("qtpy.QtSvgWidgets")

    assert QtSvgWidgets.QGraphicsSvgItem is not None
    assert QtSvgWidgets.QSvgWidget is not None



# -----------------------------------------------------------------------------
# Copyright © 2009- The Spyder Development Team
#
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
# -----------------------------------------------------------------------------

"""Provides QtSql classes and functions."""

from . import PYQT5, PYQT6, PYSIDE6, PYSIDE2, QtBindingsNotFoundError

if PYQT5:
    from PyQt5.QtSql import *
elif PYQT6:
    from PyQt6.QtSql import *
    QSqlDatabase.exec_ = lambda self, *args, **kwargs: self.exec(*args, **kwargs)
    QSqlQuery.exec_ = lambda self, *args, **kwargs: self.exec(*args, **kwargs)
    QSqlResult.exec_ = lambda self, *args, **kwargs: self.exec(*args, **kwargs)
elif PYSIDE6:
    from PySide6.QtSql import *
    # Map DeprecationWarning methods
    QSqlDatabase.exec_ = lambda self, *args, **kwargs: self.exec(*args, **kwargs)
    QSqlQuery.exec_ = lambda self, *args, **kwargs: self.exec(*args, **kwargs)
    QSqlResult.exec_ = lambda self, *args, **kwargs: self.exec(*args, **kwargs)
elif PYSIDE2:
    from PySide2.QtSql import *
else:
    raise QtBindingsNotFoundError()
